<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * Ean-13 bar code image generator
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	class EAN13_BAR_CODE	{

		# Left 6 digits code map by first number

		var $CODE_LEFT_BY_FIRST	=	array	(	0	=>	'LLLLLL',
								1	=>	'LLGLGG',
								2	=>	'LLGGLG',
								3	=>	'LLGGGL',
								4	=>	'LGLLGG',
								5	=>	'LGGLLG',
								6	=>	'LGGGLL',
								7	=>	'LGLGLG',
								8	=>	'LGLGGL',
								9	=>	'LGGLGL'	);

		# Note: Right 6 digits code map by first number is always from DigitsR 

		var $CODE_L		=	array(		0	=>	'0001101',
								1	=>	'0011001',
								2	=>	'0010011',
								3	=>	'0111101',
								4	=>	'0100011',
								5	=>	'0110001',
								6	=>	'0101111',
								7	=>	'0111011',
								8	=>	'0110111',
								9	=>	'0001011'	);

		var $CODE_G		=	array(		0	=>	'0100111',
								1	=>	'0110011',
								2	=>	'0011011',
								3	=>	'0100001',
								4	=>	'0011101',
								5	=>	'0111001',
								6	=>	'0000101',
								7 	=>	'0010001',
								8	=>	'0001001',
								9	=>	'0010111'	);

		var $CODE_R		=	array(		0	=>	'1110010',
								1	=>	'1100110',
								2	=>	'1101100',
								3	=>	'1000010',
								4	=>	'1011100',
								5	=>	'1001110',
								6	=>	'1010000',
								7	=>	'1000100',
								8	=>	'1001000',
								9	=>	'1110100'	);

		# Weights for check number calculation

		var $EAN13_WEIGHTS	=	array(		1	=>	1,
								2	=>	3,
								3	=>	1,
								4	=>	3,
								5	=>	1,
								6	=>	3,
								7	=>	1,
								8	=>	3,
								9	=>	1,
								10	=>	3,
								11	=>	1,
								12	=>	3	);

		# Left, rigth and center guard bars

		var $LEFT_RIGHT_GUARD	=	'101';

		var $CENTER_GUARD	=	'01010';

		# Bar padding

		var $PADDING		=	10;

		var $BAR_WIDTH		=	1;

		# 9 quitet + 95 bar width + 9 quitet

		var $BAR_CODE_WIDTH	=	113;

		var $BAR_CODE_HEIGHT	=	65;

		# Definition

		var $DEFINITION 	=	array();

		# Font

		var $FONT_FILE		=	FALSE;

		var $PRINT_CODE_STRING	=	FALSE;
		
		function __construct()	{
		
			if (defined('BAR_FONT_FILE')) $this->FONT_FILE = BAR_FONT_FILE;
		}

		public function code($string = FALSE)	{

			if ($string) $this->CODE = $string;
		}

		public function width($pixel = FALSE)	{

			if ((integer)$pixel > 0) $this->BAR_WIDTH = (integer)$pixel;
		}

		public function height($height = FALSE)	{

			if ($height > 0) $this->BAR_CODE_HEIGHT = $height;
		}

		public function padding($padding = FALSE)	{

			if ($padding > 0) $this->PADDING = $padding;
		}

		public function print_code()	{

			if ($this->FONT_FILE) $this->PRINT_CODE_STRING = TRUE;
		}

		# note: will be ignored, only for compatibility

		public function font_size($size) {return;}

		private function addChecksumDigit()	{

			$sum = 0;

			for ($i=11; $i>=0; $i--)	{

				$sum += $this->DIGITS[$i] * $this->EAN13_WEIGHTS[$i+1];
			}

			$checkDigit = ((intval($sum / 10) + 1) * 10) - $sum;

			if ($checkDigit == 10) $checkDigit = '0';

			$this->DIGITS[] = (string)$checkDigit;
		}

		private function defineBars()	{

			# get definition for parity in chars

			$firstCodeDef = str_split($this->CODE_LEFT_BY_FIRST[$this->DIGITS[0]]);

			# read def char by char, without the first digit which is not represent with bars

			for ($i=1; $i<13; $i++)		{

				if ($i<7)	{

					$codeBase = $this->{'CODE_'.$firstCodeDef[$i-1]};

				}	else	  {

					$codeBase = $this->CODE_R;
				}

				$this->DEFINITION [] = $codeBase[$this->DIGITS[$i]];
			}
		}

		private function find_font_size($startX, $endX)	{

			$size = $endX-$startX;

			for ($i=8; $i<50; $i++)	{

				$box = imagettfbbox($i, 0, $this->FONT_FILE, '000000');

				if ($size < ($box[2]-$box[0]))	{

					$this->FONT_SIZE = $i-3;

					break;
				}
			}
		}

		private function write_centered($startX, $endX, $string)	{

			$box = imagettfbbox($this->FONT_SIZE, 0, $this->FONT_FILE, $string);

			$diff = (($endX-$startX) - ($box[2]-$box[0])) /2;
			
			imagettftext($this->image, $this->FONT_SIZE, 0, (integer)($startX + $diff), (integer)($this->BAR_CODE_HEIGHT + $this->PADDING + $this->FONT_SIZE + 5), $this->black, $this->FONT_FILE, $string);
		}

		private function create_strings()	{

			$this->LEFT_STR = $this->RIGHT_STR = '';

			for ($i=1; $i<13; $i++)	{

				if ($i<7)	{

					$this->LEFT_STR .= $this->DIGITS[$i];

				}	else	{

					$this->RIGHT_STR .= $this->DIGITS[$i];
				}
			}
		}

		private function draw_one($definition, $guard=false)	{

			$def = str_split($definition);

			$height = $this->BAR_CODE_HEIGHT;

			if ($guard) $height += $this->BAR_GUARD_HEIGHT;

			foreach($def as $code)	{

				if ($code)	{

					imagefilledrectangle($this->image, $this->X_COOR, $this->PADDING, $this->X_COOR + $this->BAR_WIDTH -1, $height + $this->PADDING, $this->black);
				}	 

				$this->X_COOR += $this->BAR_WIDTH;
			}
		}

		private function draw_bar()	{

			$this->BAR_CODE_WIDTH	= (integer) 113 * $this->BAR_WIDTH;

			$this->BAR_GUARD_HEIGHT = (integer)(0.14 * $this->BAR_CODE_HEIGHT);

			# Create image

			if ($this->PRINT_CODE_STRING)	{

				$correction = (integer) $this->BAR_WIDTH * 8;

			}	else	  {

				$correction = 0;
			}

			$image_width	=	$this->BAR_CODE_WIDTH + 2 * $this->PADDING + $correction;

			$image_height	=	$this->BAR_CODE_HEIGHT + $this->BAR_GUARD_HEIGHT + 2 * $this->PADDING + $correction;

			$this->image = imagecreate($image_width, $image_height);

			$white = imagecolorallocate($this->image, 255, 255, 255);

			imagefill($this->image, 0, 0, $white);

			# Draw the bars

			$this->black = imagecolorallocate($this->image, 0, 0, 0);

			$this->X_COOR = $this->PADDING + 9 * $this->BAR_WIDTH + $correction; // + 9 quitet

			$STR1_X = $this->X_COOR;

			# left guard

			$this->draw_one($this->LEFT_RIGHT_GUARD, true);

			$STR2_X_S = $this->X_COOR;

			# left side

			for ($i=0; $i<6; $i++)	$this->draw_one($this->DEFINITION[$i]);

			$STR2_X_E = $this->X_COOR;

			# center guard

			$this->draw_one($this->CENTER_GUARD, true);

			$STR3_X_S = $this->X_COOR;

			# right side

			for ($i=6; $i<12; $i++)	$this->draw_one($this->DEFINITION[$i]);

			$STR3_X_E = $this->X_COOR;

			# rigth guard

			$this->draw_one($this->LEFT_RIGHT_GUARD, true);

			if ($this->PRINT_CODE_STRING)	{

				# calculate font size

				$this->find_font_size($STR2_X_S, $STR2_X_E);

				if (! $this->FONT_SIZE) return FALSE; 

				# create left and right string

				$this->create_strings();

				# write centered strings

				$this->write_centered($STR2_X_S, $STR2_X_E, $this->LEFT_STR);

				$this->write_centered($STR3_X_S, $STR3_X_E, $this->RIGHT_STR);

				# write the first digit

				imagettftext($this->image, $this->FONT_SIZE, 0, (integer)($STR1_X - 1.1 * $this->FONT_SIZE), (integer)($this->BAR_CODE_HEIGHT + $this->PADDING + $this->FONT_SIZE), $this->black, $this->FONT_FILE, $this->DIGITS[0]);
			}

			# Output

			header('Pragma: no-cache');
			header('Cache-Control: no-cache, no-store, must-revalidate');
			header('Expires: Sun, 10 Jan 2010 12:00:00 GMT');
			header('Last-Modified: '.@gmdate('D, d M Y H:i:s').' GMT');
			header('Content-type: image/png');

			imagepng($this->image);

			imagedestroy($this->image); 
		}

		public function generate()	{

			# check user input

			if (! mb_ereg_match('^[0-9]{12}+$', $this->CODE)) return FALSE;

			$this->DIGITS = str_split($this->CODE);

			# add checksum digit to end

			$this->addChecksumDigit();

			# generate definition

			$this->defineBars();

			# draw

			$this->draw_bar();

			return TRUE;
		}
	}
?>
