<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag price list print
	 */

	ob_start('ob_gzhandler');
	
	define('BASEDIR', '../');
	
	# Set time limit
	
	set_time_limit(600);

	# Turn off magic quotes runtime

	if (get_magic_quotes_runtime())	set_magic_quotes_runtime(false);

	# If magic quotes is enabled strip slashes from code

	if (get_magic_quotes_gpc())	{

		foreach ($_POST as $key=>$val) $_GET[$key] = stripslashes($val);
	}
	
	# Sets mbstring functions
	
	mb_internal_encoding('UTF-8');
	
	mb_regex_encoding('UTF-8');

	# Read configuration

	include('../classes/INIload.class.php');

	$__CONF	= new INI_load('../conf/eBag.ini');
	
	# Sets error handler

	require(BASEDIR.'classes/Errorhandler.class.php');

	$__HANDLER	=	new ERRHANDLER();
	$__HANDLER	->	start();

	# Sets session handler

	require(BASEDIR.'classes/Sessionhandler.class.php');

	$__SESS	=	new MysqlSession();
	$__SESS	->	set_session_handler();
	
	register_shutdown_function('session_write_close');

	session_start();

	# Read language
	
	include('../classes/Language.class.php');
	
	$lang 	= strtolower($__CONF->get('language', 'lang'));

	$__LANG	= new Lang("../lang/$lang/language.$lang", $__CONF->get('cache', 'use_apc_cache'));
	
	unset($lang);

	$MESSAGE = $__LANG->get('printqueryempty');

	# create connection

	$mysqli = new mysqli($__CONF->get('connection', 'host'), $__CONF->get('connection', 'user'), $__CONF->get('connection', 'password'));
	
	if ($mysqli->connect_error)	{

		include(BASEDIR.'views/printqueryempty.php');

		exit;
	}

	$mysqli->set_charset("utf8");

	# Note: collation in case sensitive because of better order by
	
	$mysqli->query("SET NAMES 'utf8' COLLATE 'utf8_unicode_ci'");
	
	if (! isset($_SESSION['BASE']))	{
	
		$JSDISABLED = $__LANG->get('jsdisabled');

		include(BASEDIR.'views/popupclose.php');
		
		exit;
	}

	if ((! isset($_POST)) or (! count($_POST)))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}
	
	extract($_POST);

	$COEF = 1;

	$IMP	=	'';

	if (isset($print_with_discount) && $print_with_discount)	{

		$COEF	=	(100-$_SESSION['discount'])/100;

		$IMP	.=	$__LANG->get('session_discount').' : '.$_SESSION['discount']." %<br>\n";
	}

	if (isset($user_defined_discount) && $user_defined_discount)	{

		$user_defined_discount = (integer)$user_defined_discount;

		if (($user_defined_discount) && ($user_defined_discount < 100))	$COEF = $COEF * (100-$user_defined_discount)/100;

		$IMP	.=	$__LANG->get('userdefineddiscount').' : '.$user_defined_discount." %<br>\n";
	}

	$COLUMNS = '';

	$COLS	=	array();

	if (isset($print_in_valuta) && $print_in_valuta) 	{

		$COLS[] = " (price * $COEF) AS price_V, (price_with_tax * $COEF) AS price_tax_V ";
	}

	if (isset($print_in_current_currency) && $print_in_current_currency) 	{

		if ($exchange_rate = $__CONF->get('payments', 'exchange_rate'))	  {

			$COEF	 = $exchange_rate * $COEF;

			$COLS[] = "  (price * $COEF) AS `price_C`, (price_with_tax * $COEF) AS `price_tax_C` ";
		}
	}
	
	if (count($COLS))	{
	  
		$COLS[] = " `tax` ";
	  
		$COLUMNS = ' ,'.implode(' , ', $COLS);
	
		unset($COLS);
	}

	$COMMAND = $PART1 = $PART2 = '';

	if (isset($include_services) && $include_services)	$PART1 = "	SELECT	`code`, `servicename` AS `name` $COLUMNS
										FROM ebagstock.services
										WHERE `deleted` IS NOT NULL ORDER BY `code`	";

	if (isset($include_articles) && $include_articles) 	$PART2 = "	SELECT	`code`, `articalname` AS `name` $COLUMNS
										FROM ebagstock.finished_goods_stock
										WHERE `deleted` IS NOT NULL ORDER BY `code`	";
	if (($PART1) && ($PART2))	{

		$COMMAND = "($PART1) UNION ALL ($PART2)";

	}	else	  {

		$COMMAND = $PART1.$PART2;
	}

	$COLUMNNAMES	=	array(  'code'		=>	$__LANG->get('code'),
					'name'		=>	$__LANG->get('name'),
					'price_C'	=>	$__LANG->get('price').'('.$__CONF->get('payments', 'currency').')',
					'price_tax_C'	=>	$__LANG->get('price_with_tax').'('.$__CONF->get('payments', 'currency').')',
					'tax'		=>	$__LANG->get('tax'),
					'price_V'	=>	$__LANG->get('price').'('.$__CONF->get('payments', 'valuta').')',
					'price_tax_V'	=>	$__LANG->get('price_with_tax').'('.$__CONF->get('payments', 'valuta').')'	);
	if ($COMMAND)	{

		$RESULT = $mysqli->query($COMMAND);

		if (! $RESULT) echo $mysqli->error;

		if ($RESULT->num_rows)		{

			$COLUMNS	=	array();

			while ($FINFO = $RESULT->fetch_field()) {
			
				$field = $FINFO->name;
				
				if (($field == 'tax') or ($field =='price_tax_C') or ($field == 'price_tax_V'))	{

					if (isset($print_prices_with_tax) && $print_prices_with_tax) array_push($COLUMNS, $FINFO->name);
					
					continue;
				}

				array_push($COLUMNS, $field);
			}

			if (count($COLUMNS))	{
			
				$VERSION_NUMBER = VERSION_NUMBER;
				
				$TEXT_DIRECTION = TEXT_DIRECTION;

				echo	<<<"EOT"
					<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
					<html>
						<head>
							<meta http-equiv="Content-Type" CONTENT="text/html; charset=utf-8">
					 		<meta name="robots" content="noindex,nofollow">
							<title>eBag</title>
							<link href="../css/$TEXT_DIRECTION/tableprint.css?ver=$VERSION_NUMBER" rel="stylesheet" type="text/css" media="print,screen">
						</head>
					<body>
EOT;

				echo "<table>\n";

				include (BASEDIR.'functions/mb_wrap.php');

				echo "<thead><tr>\n";

				foreach ($COLUMNS as $COLUMN)	echo "<td class=\"bottom\">".mb_wrap($COLUMNNAMES[$COLUMN], 7, '<br>')."</td>\n";

				echo "</tr></thead><tbody>\n";

				$inc	=	0;

				ob_start();

				while ($ROW = $RESULT->fetch_assoc())	{

					echo "<tr>\n";

					foreach ($COLUMNS as $COLUMN)	echo '<td>'.$ROW[$COLUMN].'</td>';

					echo "</tr>\n";

					$inc++;

					if ($inc == 100)	{

						ob_end_flush();
						ob_start();
						$inc = 0;
					}
				}
	
				echo "</tbody></table></body></html>\n";

				echo '<br><br>'.$IMP;

				ob_end_flush();
				
				exit;
			}
		}
	}

	include(BASEDIR.'views/printqueryempty.php');
?>
