<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 5.0.0
	 */

	/**
	 * eBag write new production order
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('date2sql');

	load_f('is_real_positive');

	load_f('is_int_positive');


	class PRODUCTIONORDERWRITE	{

		var 	$ORDER_TO_STOCK		=	FALSE;

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($this->CONF->get('cache', 'use_apc_cache'));

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';

			$__PHASES		=	new PHASEloader($this->CONF->get('cache', 'use_apc_cache'));

			$this->DEFAULT_PHASE	=	$this->SQL->n_escape($__PHASES->getdefault());

			unset($__PHASES);
		}

		public function getid()		{

			if (isset($this->productionorderID)) {return $this->productionorderID;} else {return FALSE;}
		}

		private function lock()	{

			$this->SQL->query("	LOCK TABLES 	$this->BASE.production_orders_header WRITE, 
								$this->BASE.production_orders_data WRITE, 
								$this->BASE.production_orders_phases WRITE, 
								$this->BASE.orders_data WRITE,
								$this->STOCKBASE.feedstock WRITE, 
								$this->STOCKBASE.finished_goods_stock WRITE		");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)	{

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];
			$TYPES		=	$DATA['type'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];
				$type		=	$TYPES[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;
				
				# initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity']))	$RETURN['quantity']= array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();
				if (! isset($RETURN['type'])) 		$RETURN['type'] = array();

				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}				

				if ($RETURN == 'usererror') break;

				# feedstock or artical exists
				
				if ($type == 'a') {$stock = 'finished_goods_stock'; $return_str = 'articalnotexist';} else {$stock = 'feedstock'; $return_str = 'feedstocknotexist';}

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `personID` 
							FROM $this->STOCKBASE.$stock
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
				
				if (! $this->SQL->num_rows()) {$RETURN = $return_str; break;}

				$RETURN['name'][]	=	$this->SQL->n_escape($name);
				$RETURN['quantity'][]	=	$quantity;
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
				$RETURN['type'][]	=	$this->SQL->n_escape($type);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;			
		}

		public function write($DATA)	{

			# collect header data

			$articalcode		=	$this->SQL->n_escape($DATA['articalcode']);

			$articalname		=	$this->SQL->n_escape($DATA['articalname']);

			$orderID		=	$this->SQL->escape($DATA['orderID']);

			$rowID			=	$this->SQL->escape($DATA['rowID']);
			
			$contactname		=	$this->SQL->n_escape($DATA['contactname']);

			$date_of_creation	=	$this->SQL->n_escape(date2sql($DATA['date_of_creation'], $this->CONF->get('date', 'dateformat')));

			$date_of_deadline	=	$this->SQL->n_escape(date2sql($DATA['date_of_deadline'], $this->CONF->get('date', 'dateformat')));

			$technology		=	$this->SQL->n_escape($DATA['technology']);

			$drawingID		=	$this->SQL->n_escape($DATA['drawingID']);

			$this->SQL->transaction();
		
			$this->lock();

			# check the order is not changed or deleted

			$this->SQL->query("	SELECT `quantity` 
						FROM $this->BASE.orders_data 
						WHERE `personID`='$rowID' AND `deleted` IS NOT NULL LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentorder';
			}

			# reread the quantity to be safe if is changed

			$RESULT			=	$this->SQL->RESULT->fetch_row();

			$articalquantity	=	$RESULT[0];

			# prepare (feedstocks) data

			$DATA = $this->prepare($DATA);

			if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}
		
			# insert production order header 
			# note : stocktypes 1=>finished_goods_stock, 2=>operating_suplies
			
			$stocktype = 1;

			$this->SQL->query("	INSERT INTO $this->BASE.production_orders_header 
						VALUES 
						(NULL, '$stocktype', $articalcode, $articalname, $contactname, '$orderID', $date_of_creation, 
						$date_of_deadline, '0', NULL, '$articalquantity', '0', $drawingID, $technology, 
						$this->DEFAULT_PHASE, NULL, '0')	");

			$this->productionorderID = $this->SQL->lastid();

			# update the stock

			$this->SQL->query("	UPDATE $this->STOCKBASE.finished_goods_stock 
						SET quantity_in_production=quantity_in_production+$articalquantity
						WHERE `code`=$articalcode AND `deleted` IS NOT NULL LIMIT 1	");

			$COUNT		=	count($DATA['code']);

			for ($i=0; $i<$COUNT; $i++)	{

				$code		=	$DATA['code'][$i];
				$name		=	$DATA['name'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$unit		=	$DATA['unit'][$i];
				$type		=	$DATA['type'][$i];

				# check for same code

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.production_orders_data 
							WHERE `code`=$code AND `productionorderID`='$this->productionorderID' AND `type`=$type AND `deleted` IS NOT NULL LIMIT 1		");

				if ($this->SQL->num_rows())	{

					# update

					$this->SQL->query("	UPDATE $this->BASE.production_orders_data 
								SET quantity=quantity+$quantity
								WHERE `code`=$code AND `productionorderID`='$this->productionorderID' AND `type`=$type AND `deleted` IS NOT NULL LIMIT 1		");
				}	else	  {

					# insert new one

					$this->SQL->query("	INSERT INTO $this->BASE.production_orders_data 
								VALUES  
								(NULL, '$this->productionorderID', $code, $name, $unit, '$quantity', $type, '0')	");
				}

				# check stock state and set to order

				$SUM	=	(double)$articalquantity*(double)$quantity; 

				if (! $this->ORDER_TO_STOCK)	{
				
					if ($type == "'a'")	{
					
						# finished goods stock
						# note: origin 1 is product which will not be ordered
						
						$this->SQL->query("	SELECT	quantity+quantity_ordered_to_stock+quantity_on_opened_order_to_stock-quantity_ordered-quantity_as_part-$SUM
									FROM $this->STOCKBASE.finished_goods_stock 
									WHERE `code`=$code AND `origin`!='1' AND `deleted` IS NOT NULL LIMIT 1	");

						if ($this->SQL->num_rows())	{

							$RESULT	= $this->SQL->RESULT->fetch_row();
							
							if ($RESULT[0] < 0) $this->ORDER_TO_STOCK = TRUE;
						}
						  
					}	else	{
					
						# feedstock
						
						$this->SQL->query("	SELECT quantity+quantity_ordered_to_stock+quantity_on_opened_order_to_stock-$SUM
									FROM $this->STOCKBASE.feedstock 
									WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

						# note: must to be row, checked before

						$RESULT	= $this->SQL->RESULT->fetch_row();

						if ($RESULT[0] < 0) $this->ORDER_TO_STOCK = TRUE;
					}
				}
				
				if ($type == "'a'")	{
				
					# update finished goods stock
				
					# Note : finished goods stock final quantity will be updated when the production order is 
					#	 closed now update only quantity used as part
					
					$this->SQL->query("	UPDATE $this->STOCKBASE.finished_goods_stock
								SET quantity_as_part=quantity_as_part+$SUM
								WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
				}	else	 {
				
					# update feedstock

					# Note: the feedstock quantity can be negative,
					#	other case will cause that the production orders cannot be created if you not have
					#	feedstock, and you cannot to work (example: making a whole project without ordering yet)

					$this->SQL->query("	UPDATE $this->STOCKBASE.feedstock
								SET quantity_in_production=quantity_in_production+$SUM, quantity=quantity-$SUM
								WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
				}
			}
			
			# update orders data set production order to this artical

			$this->SQL->query("	UPDATE $this->BASE.orders_data 
						SET `productionorderID`='$this->productionorderID' 
						WHERE `personID`='$rowID' LIMIT 1	");


			# set the first phase of production -> feedstock

			$this->SQL->query("	INSERT INTO $this->BASE.production_orders_phases 
						VALUES 
						(NULL, '$this->productionorderID', $this->DEFAULT_PHASE, NULL, NULL, NULL)	");

			$this->SQL->commit();

			$this->unlock();

			return TRUE;			
		}
	}

?>
